public class Matrix {
    private int line;
    private int col; // Colonnes
    private double[][] data;

    public int getLine() {
        return line;
    }

    public int getCol() {
        return col;
    }

    public double getData(int line, int col) {
        return data[line][col];
    }

    public double[][] getDatas() {
        return data;
    }

    public void setData(int line, int col, double d) {
        this.data[line][col] = d;
    }

    /**
     * Constructeur d'une matrice
     *
     * @param line le nombre de lignes
     * @param col  le nombre de colonnes
     */
    public Matrix(int line, int col) {
        this.line = line;
        this.col = col;
        this.data = new double[line][col];
    }

    /**
     * Remplir la matrice grâce à une autre matrice
     *
     * @param line nombre de lignes de la matrice passée en paramètre
     * @param col  nombre de colonnes de la matrice passée en paramètre
     * @param tab  la matrice qui sert à remplir celle de la classe
     * @throws IndexOutOfBoundsException Si matrice passée en paramètre en + grande que celle de la classe
     */
    public void matrixFill(int line, int col, double[][] tab) throws IndexOutOfBoundsException {
        // Parcourir
        for (int i = 0; i < line; i++) {
            for (int j = 0; j < col; j++) {
                // Si index contenu dans la matrice de la classe, ajouter la valeur dans celle-ci
                if (i < tab.length && j < tab[i].length)
                    this.data[i][j] = tab[i][j];
            }
        }
    }

    /**
     * Réallouer la matrice (changer le nombre de ligne(s) / colonne(s))
     *
     * @param line nombre de lignes
     * @param col  nombre de colonnes
     */
    public void matrixRealloc(int line, int col) {
        // Stocker temporairement les données
        double[][] tmp = this.data;
        // Réallouer le nb de ligne(s) / colonne(s)
        this.line = line;
        this.col = col;
        this.data = new double[line][col];
        // Remplir la nouvelle matrice avec les anciennes valeurs (le reste à 0)
        matrixFill(line, col, tmp);
    }

    /**
     * Print une matrice
     *
     * @param s Titre d'une matrice
     */
    public void matrixPrint(String s) {
        // Titre
        System.out.println();
        System.out.println(s + ": ");
        // Parcourir la matrice et afficher les valeurs arrondies au centième
        for (int i = 0; i < line; i++) {
            for (int j = 0; j < col; j++) {
                System.out.format("%10.2f", this.data[i][j]);
            }
            System.out.println();
        }
    }

    /**
     * Print du tableau auxiliaire comme dans l'énnoncé
     * @param s le titre
     * @param nbX nombre de X
     * @param nbZ nombre de Z
     * @param nbAux nombre d'auxiliaires
     */
    public void printTabAux(String s, int nbX, int nbZ, int nbAux) {
        System.out.println("---------------------------");
        System.out.println(s);
        System.out.println("---------------------------");
        System.out.print("COLS:              ");
        for (int i = 0; i < nbX; i++) {
            System.out.format("X[%d]      ", i);
        }

        for (int i = 0; i < nbZ; i++) {
            if (i < 10)
                System.out.format("Z[%d]      ", i);
            else
                System.out.format("Z[%d]     ", i);
        }

        System.out.format("S[%d]     ", this.col / 2);

        for (int i = 0; i < nbAux; i++) {
            if (i < 10)
                System.out.format("A[%d]      ", i);
            else
                System.out.format("A[%d]     ", i);
        }
        System.out.println();

        for (int i = 0; i < this.line; i++) {
            if (i < this.line - 2)
                if (i < 10)
                    System.out.print("AUX_" + i + "     |   ");
                else
                    System.out.print("AUX_" + i + "    |   ");
            else if (i == this.line - 2)
                System.out.print("OBJ. Init |   ");
            else System.out.print("OBJ.      |   ");
            for (int j = 0; j < this.col; j++) {
                System.out.format("%10.3f", this.data[i][j]);
            }
            System.out.println();
        }
    }
}
